// https://www.terraform.io/docs/providers/google/d/container_cluster.html
// generated from terraform resource schema

import { Construct } from 'constructs';
import * as cdktf from 'cdktf';

// Configuration

export interface DataGoogleContainerClusterConfig extends cdktf.TerraformMetaArguments {
  /**
  * The location (region or zone) in which the cluster master will be created, as well as the default node location. If you specify a zone (such as us-central1-a), the cluster will be a zonal cluster with a single cluster master. If you specify a region (such as us-west1), the cluster will be a regional cluster with multiple masters spread across zones in the region, and with default node locations in those zones as well.
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/d/container_cluster.html#location DataGoogleContainerCluster#location}
  */
  readonly location?: string;
  /**
  * The name of the cluster, unique within the project and location.
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/d/container_cluster.html#name DataGoogleContainerCluster#name}
  */
  readonly name: string;
  /**
  * The ID of the project in which the resource belongs. If it is not provided, the provider project is used.
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/d/container_cluster.html#project DataGoogleContainerCluster#project}
  */
  readonly project?: string;
}
export class DataGoogleContainerClusterAddonsConfigCloudrunConfig extends cdktf.ComplexComputedList {

  // disabled - computed: true, optional: false, required: false
  public get disabled() {
    return this.getBooleanAttribute('disabled') as any;
  }

  // load_balancer_type - computed: true, optional: false, required: false
  public get loadBalancerType() {
    return this.getStringAttribute('load_balancer_type');
  }
}
export class DataGoogleContainerClusterAddonsConfigGcpFilestoreCsiDriverConfig extends cdktf.ComplexComputedList {

  // enabled - computed: true, optional: false, required: false
  public get enabled() {
    return this.getBooleanAttribute('enabled') as any;
  }
}
export class DataGoogleContainerClusterAddonsConfigHorizontalPodAutoscaling extends cdktf.ComplexComputedList {

  // disabled - computed: true, optional: false, required: false
  public get disabled() {
    return this.getBooleanAttribute('disabled') as any;
  }
}
export class DataGoogleContainerClusterAddonsConfigHttpLoadBalancing extends cdktf.ComplexComputedList {

  // disabled - computed: true, optional: false, required: false
  public get disabled() {
    return this.getBooleanAttribute('disabled') as any;
  }
}
export class DataGoogleContainerClusterAddonsConfigNetworkPolicyConfig extends cdktf.ComplexComputedList {

  // disabled - computed: true, optional: false, required: false
  public get disabled() {
    return this.getBooleanAttribute('disabled') as any;
  }
}
export class DataGoogleContainerClusterAddonsConfig extends cdktf.ComplexComputedList {

  // cloudrun_config - computed: true, optional: false, required: false
  public get cloudrunConfig() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('cloudrun_config') as any;
  }

  // gcp_filestore_csi_driver_config - computed: true, optional: false, required: false
  public get gcpFilestoreCsiDriverConfig() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('gcp_filestore_csi_driver_config') as any;
  }

  // horizontal_pod_autoscaling - computed: true, optional: false, required: false
  public get horizontalPodAutoscaling() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('horizontal_pod_autoscaling') as any;
  }

  // http_load_balancing - computed: true, optional: false, required: false
  public get httpLoadBalancing() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('http_load_balancing') as any;
  }

  // network_policy_config - computed: true, optional: false, required: false
  public get networkPolicyConfig() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('network_policy_config') as any;
  }
}
export class DataGoogleContainerClusterAuthenticatorGroupsConfig extends cdktf.ComplexComputedList {

  // security_group - computed: true, optional: false, required: false
  public get securityGroup() {
    return this.getStringAttribute('security_group');
  }
}
export class DataGoogleContainerClusterClusterAutoscalingAutoProvisioningDefaults extends cdktf.ComplexComputedList {

  // image_type - computed: true, optional: false, required: false
  public get imageType() {
    return this.getStringAttribute('image_type');
  }

  // oauth_scopes - computed: true, optional: false, required: false
  public get oauthScopes() {
    return this.getListAttribute('oauth_scopes');
  }

  // service_account - computed: true, optional: false, required: false
  public get serviceAccount() {
    return this.getStringAttribute('service_account');
  }
}
export class DataGoogleContainerClusterClusterAutoscalingResourceLimits extends cdktf.ComplexComputedList {

  // maximum - computed: true, optional: false, required: false
  public get maximum() {
    return this.getNumberAttribute('maximum');
  }

  // minimum - computed: true, optional: false, required: false
  public get minimum() {
    return this.getNumberAttribute('minimum');
  }

  // resource_type - computed: true, optional: false, required: false
  public get resourceType() {
    return this.getStringAttribute('resource_type');
  }
}
export class DataGoogleContainerClusterClusterAutoscaling extends cdktf.ComplexComputedList {

  // auto_provisioning_defaults - computed: true, optional: false, required: false
  public get autoProvisioningDefaults() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('auto_provisioning_defaults') as any;
  }

  // enabled - computed: true, optional: false, required: false
  public get enabled() {
    return this.getBooleanAttribute('enabled') as any;
  }

  // resource_limits - computed: true, optional: false, required: false
  public get resourceLimits() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('resource_limits') as any;
  }
}
export class DataGoogleContainerClusterConfidentialNodes extends cdktf.ComplexComputedList {

  // enabled - computed: true, optional: false, required: false
  public get enabled() {
    return this.getBooleanAttribute('enabled') as any;
  }
}
export class DataGoogleContainerClusterDatabaseEncryption extends cdktf.ComplexComputedList {

  // key_name - computed: true, optional: false, required: false
  public get keyName() {
    return this.getStringAttribute('key_name');
  }

  // state - computed: true, optional: false, required: false
  public get state() {
    return this.getStringAttribute('state');
  }
}
export class DataGoogleContainerClusterDefaultSnatStatus extends cdktf.ComplexComputedList {

  // disabled - computed: true, optional: false, required: false
  public get disabled() {
    return this.getBooleanAttribute('disabled') as any;
  }
}
export class DataGoogleContainerClusterDnsConfig extends cdktf.ComplexComputedList {

  // cluster_dns - computed: true, optional: false, required: false
  public get clusterDns() {
    return this.getStringAttribute('cluster_dns');
  }

  // cluster_dns_domain - computed: true, optional: false, required: false
  public get clusterDnsDomain() {
    return this.getStringAttribute('cluster_dns_domain');
  }

  // cluster_dns_scope - computed: true, optional: false, required: false
  public get clusterDnsScope() {
    return this.getStringAttribute('cluster_dns_scope');
  }
}
export class DataGoogleContainerClusterIpAllocationPolicy extends cdktf.ComplexComputedList {

  // cluster_ipv4_cidr_block - computed: true, optional: false, required: false
  public get clusterIpv4CidrBlock() {
    return this.getStringAttribute('cluster_ipv4_cidr_block');
  }

  // cluster_secondary_range_name - computed: true, optional: false, required: false
  public get clusterSecondaryRangeName() {
    return this.getStringAttribute('cluster_secondary_range_name');
  }

  // services_ipv4_cidr_block - computed: true, optional: false, required: false
  public get servicesIpv4CidrBlock() {
    return this.getStringAttribute('services_ipv4_cidr_block');
  }

  // services_secondary_range_name - computed: true, optional: false, required: false
  public get servicesSecondaryRangeName() {
    return this.getStringAttribute('services_secondary_range_name');
  }
}
export class DataGoogleContainerClusterLoggingConfig extends cdktf.ComplexComputedList {

  // enable_components - computed: true, optional: false, required: false
  public get enableComponents() {
    return this.getListAttribute('enable_components');
  }
}
export class DataGoogleContainerClusterMaintenancePolicyDailyMaintenanceWindow extends cdktf.ComplexComputedList {

  // duration - computed: true, optional: false, required: false
  public get duration() {
    return this.getStringAttribute('duration');
  }

  // start_time - computed: true, optional: false, required: false
  public get startTime() {
    return this.getStringAttribute('start_time');
  }
}
export class DataGoogleContainerClusterMaintenancePolicyMaintenanceExclusionExclusionOptions extends cdktf.ComplexComputedList {

  // scope - computed: true, optional: false, required: false
  public get scope() {
    return this.getStringAttribute('scope');
  }
}
export class DataGoogleContainerClusterMaintenancePolicyMaintenanceExclusion extends cdktf.ComplexComputedList {

  // end_time - computed: true, optional: false, required: false
  public get endTime() {
    return this.getStringAttribute('end_time');
  }

  // exclusion_name - computed: true, optional: false, required: false
  public get exclusionName() {
    return this.getStringAttribute('exclusion_name');
  }

  // exclusion_options - computed: true, optional: false, required: false
  public get exclusionOptions() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('exclusion_options') as any;
  }

  // start_time - computed: true, optional: false, required: false
  public get startTime() {
    return this.getStringAttribute('start_time');
  }
}
export class DataGoogleContainerClusterMaintenancePolicyRecurringWindow extends cdktf.ComplexComputedList {

  // end_time - computed: true, optional: false, required: false
  public get endTime() {
    return this.getStringAttribute('end_time');
  }

  // recurrence - computed: true, optional: false, required: false
  public get recurrence() {
    return this.getStringAttribute('recurrence');
  }

  // start_time - computed: true, optional: false, required: false
  public get startTime() {
    return this.getStringAttribute('start_time');
  }
}
export class DataGoogleContainerClusterMaintenancePolicy extends cdktf.ComplexComputedList {

  // daily_maintenance_window - computed: true, optional: false, required: false
  public get dailyMaintenanceWindow() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('daily_maintenance_window') as any;
  }

  // maintenance_exclusion - computed: true, optional: false, required: false
  public get maintenanceExclusion() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('maintenance_exclusion') as any;
  }

  // recurring_window - computed: true, optional: false, required: false
  public get recurringWindow() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('recurring_window') as any;
  }
}
export class DataGoogleContainerClusterMasterAuthClientCertificateConfig extends cdktf.ComplexComputedList {

  // issue_client_certificate - computed: true, optional: false, required: false
  public get issueClientCertificate() {
    return this.getBooleanAttribute('issue_client_certificate') as any;
  }
}
export class DataGoogleContainerClusterMasterAuth extends cdktf.ComplexComputedList {

  // client_certificate - computed: true, optional: false, required: false
  public get clientCertificate() {
    return this.getStringAttribute('client_certificate');
  }

  // client_certificate_config - computed: true, optional: false, required: false
  public get clientCertificateConfig() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('client_certificate_config') as any;
  }

  // client_key - computed: true, optional: false, required: false
  public get clientKey() {
    return this.getStringAttribute('client_key');
  }

  // cluster_ca_certificate - computed: true, optional: false, required: false
  public get clusterCaCertificate() {
    return this.getStringAttribute('cluster_ca_certificate');
  }
}
export class DataGoogleContainerClusterMasterAuthorizedNetworksConfigCidrBlocks extends cdktf.ComplexComputedList {

  // cidr_block - computed: true, optional: false, required: false
  public get cidrBlock() {
    return this.getStringAttribute('cidr_block');
  }

  // display_name - computed: true, optional: false, required: false
  public get displayName() {
    return this.getStringAttribute('display_name');
  }
}
export class DataGoogleContainerClusterMasterAuthorizedNetworksConfig extends cdktf.ComplexComputedList {

  // cidr_blocks - computed: true, optional: false, required: false
  public get cidrBlocks() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('cidr_blocks') as any;
  }
}
export class DataGoogleContainerClusterMonitoringConfig extends cdktf.ComplexComputedList {

  // enable_components - computed: true, optional: false, required: false
  public get enableComponents() {
    return this.getListAttribute('enable_components');
  }
}
export class DataGoogleContainerClusterNetworkPolicy extends cdktf.ComplexComputedList {

  // enabled - computed: true, optional: false, required: false
  public get enabled() {
    return this.getBooleanAttribute('enabled') as any;
  }

  // provider - computed: true, optional: false, required: false
  public get provider() {
    return this.getStringAttribute('provider');
  }
}
export class DataGoogleContainerClusterNodeConfigGcfsConfig extends cdktf.ComplexComputedList {

  // enabled - computed: true, optional: false, required: false
  public get enabled() {
    return this.getBooleanAttribute('enabled') as any;
  }
}
export class DataGoogleContainerClusterNodeConfigGuestAccelerator extends cdktf.ComplexComputedList {

  // count - computed: true, optional: false, required: false
  public get count() {
    return this.getNumberAttribute('count');
  }

  // gpu_partition_size - computed: true, optional: false, required: false
  public get gpuPartitionSize() {
    return this.getStringAttribute('gpu_partition_size');
  }

  // type - computed: true, optional: false, required: false
  public get type() {
    return this.getStringAttribute('type');
  }
}
export class DataGoogleContainerClusterNodeConfigGvnic extends cdktf.ComplexComputedList {

  // enabled - computed: true, optional: false, required: false
  public get enabled() {
    return this.getBooleanAttribute('enabled') as any;
  }
}
export class DataGoogleContainerClusterNodeConfigShieldedInstanceConfig extends cdktf.ComplexComputedList {

  // enable_integrity_monitoring - computed: true, optional: false, required: false
  public get enableIntegrityMonitoring() {
    return this.getBooleanAttribute('enable_integrity_monitoring') as any;
  }

  // enable_secure_boot - computed: true, optional: false, required: false
  public get enableSecureBoot() {
    return this.getBooleanAttribute('enable_secure_boot') as any;
  }
}
export class DataGoogleContainerClusterNodeConfigTaint extends cdktf.ComplexComputedList {

  // effect - computed: true, optional: false, required: false
  public get effect() {
    return this.getStringAttribute('effect');
  }

  // key - computed: true, optional: false, required: false
  public get key() {
    return this.getStringAttribute('key');
  }

  // value - computed: true, optional: false, required: false
  public get value() {
    return this.getStringAttribute('value');
  }
}
export class DataGoogleContainerClusterNodeConfigWorkloadMetadataConfig extends cdktf.ComplexComputedList {

  // mode - computed: true, optional: false, required: false
  public get mode() {
    return this.getStringAttribute('mode');
  }
}
export class DataGoogleContainerClusterNodeConfig extends cdktf.ComplexComputedList {

  // boot_disk_kms_key - computed: true, optional: false, required: false
  public get bootDiskKmsKey() {
    return this.getStringAttribute('boot_disk_kms_key');
  }

  // disk_size_gb - computed: true, optional: false, required: false
  public get diskSizeGb() {
    return this.getNumberAttribute('disk_size_gb');
  }

  // disk_type - computed: true, optional: false, required: false
  public get diskType() {
    return this.getStringAttribute('disk_type');
  }

  // gcfs_config - computed: true, optional: false, required: false
  public get gcfsConfig() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('gcfs_config') as any;
  }

  // guest_accelerator - computed: true, optional: false, required: false
  public get guestAccelerator() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('guest_accelerator') as any;
  }

  // gvnic - computed: true, optional: false, required: false
  public get gvnic() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('gvnic') as any;
  }

  // image_type - computed: true, optional: false, required: false
  public get imageType() {
    return this.getStringAttribute('image_type');
  }

  // labels - computed: true, optional: false, required: false
  public get labels() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('labels') as any;
  }

  // local_ssd_count - computed: true, optional: false, required: false
  public get localSsdCount() {
    return this.getNumberAttribute('local_ssd_count');
  }

  // machine_type - computed: true, optional: false, required: false
  public get machineType() {
    return this.getStringAttribute('machine_type');
  }

  // metadata - computed: true, optional: false, required: false
  public get metadata() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('metadata') as any;
  }

  // min_cpu_platform - computed: true, optional: false, required: false
  public get minCpuPlatform() {
    return this.getStringAttribute('min_cpu_platform');
  }

  // node_group - computed: true, optional: false, required: false
  public get nodeGroup() {
    return this.getStringAttribute('node_group');
  }

  // oauth_scopes - computed: true, optional: false, required: false
  public get oauthScopes() {
    return this.getListAttribute('oauth_scopes');
  }

  // preemptible - computed: true, optional: false, required: false
  public get preemptible() {
    return this.getBooleanAttribute('preemptible') as any;
  }

  // service_account - computed: true, optional: false, required: false
  public get serviceAccount() {
    return this.getStringAttribute('service_account');
  }

  // shielded_instance_config - computed: true, optional: false, required: false
  public get shieldedInstanceConfig() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('shielded_instance_config') as any;
  }

  // spot - computed: true, optional: false, required: false
  public get spot() {
    return this.getBooleanAttribute('spot') as any;
  }

  // tags - computed: true, optional: false, required: false
  public get tags() {
    return this.getListAttribute('tags');
  }

  // taint - computed: true, optional: false, required: false
  public get taint() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('taint') as any;
  }

  // workload_metadata_config - computed: true, optional: false, required: false
  public get workloadMetadataConfig() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('workload_metadata_config') as any;
  }
}
export class DataGoogleContainerClusterNodePoolAutoscaling extends cdktf.ComplexComputedList {

  // max_node_count - computed: true, optional: false, required: false
  public get maxNodeCount() {
    return this.getNumberAttribute('max_node_count');
  }

  // min_node_count - computed: true, optional: false, required: false
  public get minNodeCount() {
    return this.getNumberAttribute('min_node_count');
  }
}
export class DataGoogleContainerClusterNodePoolManagement extends cdktf.ComplexComputedList {

  // auto_repair - computed: true, optional: false, required: false
  public get autoRepair() {
    return this.getBooleanAttribute('auto_repair') as any;
  }

  // auto_upgrade - computed: true, optional: false, required: false
  public get autoUpgrade() {
    return this.getBooleanAttribute('auto_upgrade') as any;
  }
}
export class DataGoogleContainerClusterNodePoolNodeConfigGcfsConfig extends cdktf.ComplexComputedList {

  // enabled - computed: true, optional: false, required: false
  public get enabled() {
    return this.getBooleanAttribute('enabled') as any;
  }
}
export class DataGoogleContainerClusterNodePoolNodeConfigGuestAccelerator extends cdktf.ComplexComputedList {

  // count - computed: true, optional: false, required: false
  public get count() {
    return this.getNumberAttribute('count');
  }

  // gpu_partition_size - computed: true, optional: false, required: false
  public get gpuPartitionSize() {
    return this.getStringAttribute('gpu_partition_size');
  }

  // type - computed: true, optional: false, required: false
  public get type() {
    return this.getStringAttribute('type');
  }
}
export class DataGoogleContainerClusterNodePoolNodeConfigGvnic extends cdktf.ComplexComputedList {

  // enabled - computed: true, optional: false, required: false
  public get enabled() {
    return this.getBooleanAttribute('enabled') as any;
  }
}
export class DataGoogleContainerClusterNodePoolNodeConfigShieldedInstanceConfig extends cdktf.ComplexComputedList {

  // enable_integrity_monitoring - computed: true, optional: false, required: false
  public get enableIntegrityMonitoring() {
    return this.getBooleanAttribute('enable_integrity_monitoring') as any;
  }

  // enable_secure_boot - computed: true, optional: false, required: false
  public get enableSecureBoot() {
    return this.getBooleanAttribute('enable_secure_boot') as any;
  }
}
export class DataGoogleContainerClusterNodePoolNodeConfigTaint extends cdktf.ComplexComputedList {

  // effect - computed: true, optional: false, required: false
  public get effect() {
    return this.getStringAttribute('effect');
  }

  // key - computed: true, optional: false, required: false
  public get key() {
    return this.getStringAttribute('key');
  }

  // value - computed: true, optional: false, required: false
  public get value() {
    return this.getStringAttribute('value');
  }
}
export class DataGoogleContainerClusterNodePoolNodeConfigWorkloadMetadataConfig extends cdktf.ComplexComputedList {

  // mode - computed: true, optional: false, required: false
  public get mode() {
    return this.getStringAttribute('mode');
  }
}
export class DataGoogleContainerClusterNodePoolNodeConfig extends cdktf.ComplexComputedList {

  // boot_disk_kms_key - computed: true, optional: false, required: false
  public get bootDiskKmsKey() {
    return this.getStringAttribute('boot_disk_kms_key');
  }

  // disk_size_gb - computed: true, optional: false, required: false
  public get diskSizeGb() {
    return this.getNumberAttribute('disk_size_gb');
  }

  // disk_type - computed: true, optional: false, required: false
  public get diskType() {
    return this.getStringAttribute('disk_type');
  }

  // gcfs_config - computed: true, optional: false, required: false
  public get gcfsConfig() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('gcfs_config') as any;
  }

  // guest_accelerator - computed: true, optional: false, required: false
  public get guestAccelerator() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('guest_accelerator') as any;
  }

  // gvnic - computed: true, optional: false, required: false
  public get gvnic() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('gvnic') as any;
  }

  // image_type - computed: true, optional: false, required: false
  public get imageType() {
    return this.getStringAttribute('image_type');
  }

  // labels - computed: true, optional: false, required: false
  public get labels() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('labels') as any;
  }

  // local_ssd_count - computed: true, optional: false, required: false
  public get localSsdCount() {
    return this.getNumberAttribute('local_ssd_count');
  }

  // machine_type - computed: true, optional: false, required: false
  public get machineType() {
    return this.getStringAttribute('machine_type');
  }

  // metadata - computed: true, optional: false, required: false
  public get metadata() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('metadata') as any;
  }

  // min_cpu_platform - computed: true, optional: false, required: false
  public get minCpuPlatform() {
    return this.getStringAttribute('min_cpu_platform');
  }

  // node_group - computed: true, optional: false, required: false
  public get nodeGroup() {
    return this.getStringAttribute('node_group');
  }

  // oauth_scopes - computed: true, optional: false, required: false
  public get oauthScopes() {
    return this.getListAttribute('oauth_scopes');
  }

  // preemptible - computed: true, optional: false, required: false
  public get preemptible() {
    return this.getBooleanAttribute('preemptible') as any;
  }

  // service_account - computed: true, optional: false, required: false
  public get serviceAccount() {
    return this.getStringAttribute('service_account');
  }

  // shielded_instance_config - computed: true, optional: false, required: false
  public get shieldedInstanceConfig() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('shielded_instance_config') as any;
  }

  // spot - computed: true, optional: false, required: false
  public get spot() {
    return this.getBooleanAttribute('spot') as any;
  }

  // tags - computed: true, optional: false, required: false
  public get tags() {
    return this.getListAttribute('tags');
  }

  // taint - computed: true, optional: false, required: false
  public get taint() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('taint') as any;
  }

  // workload_metadata_config - computed: true, optional: false, required: false
  public get workloadMetadataConfig() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('workload_metadata_config') as any;
  }
}
export class DataGoogleContainerClusterNodePoolUpgradeSettings extends cdktf.ComplexComputedList {

  // max_surge - computed: true, optional: false, required: false
  public get maxSurge() {
    return this.getNumberAttribute('max_surge');
  }

  // max_unavailable - computed: true, optional: false, required: false
  public get maxUnavailable() {
    return this.getNumberAttribute('max_unavailable');
  }
}
export class DataGoogleContainerClusterNodePool extends cdktf.ComplexComputedList {

  // autoscaling - computed: true, optional: false, required: false
  public get autoscaling() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('autoscaling') as any;
  }

  // initial_node_count - computed: true, optional: false, required: false
  public get initialNodeCount() {
    return this.getNumberAttribute('initial_node_count');
  }

  // instance_group_urls - computed: true, optional: false, required: false
  public get instanceGroupUrls() {
    return this.getListAttribute('instance_group_urls');
  }

  // managed_instance_group_urls - computed: true, optional: false, required: false
  public get managedInstanceGroupUrls() {
    return this.getListAttribute('managed_instance_group_urls');
  }

  // management - computed: true, optional: false, required: false
  public get management() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('management') as any;
  }

  // max_pods_per_node - computed: true, optional: false, required: false
  public get maxPodsPerNode() {
    return this.getNumberAttribute('max_pods_per_node');
  }

  // name - computed: true, optional: false, required: false
  public get name() {
    return this.getStringAttribute('name');
  }

  // name_prefix - computed: true, optional: false, required: false
  public get namePrefix() {
    return this.getStringAttribute('name_prefix');
  }

  // node_config - computed: true, optional: false, required: false
  public get nodeConfig() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('node_config') as any;
  }

  // node_count - computed: true, optional: false, required: false
  public get nodeCount() {
    return this.getNumberAttribute('node_count');
  }

  // node_locations - computed: true, optional: false, required: false
  public get nodeLocations() {
    return this.getListAttribute('node_locations');
  }

  // upgrade_settings - computed: true, optional: false, required: false
  public get upgradeSettings() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('upgrade_settings') as any;
  }

  // version - computed: true, optional: false, required: false
  public get version() {
    return this.getStringAttribute('version');
  }
}
export class DataGoogleContainerClusterPrivateClusterConfigMasterGlobalAccessConfig extends cdktf.ComplexComputedList {

  // enabled - computed: true, optional: false, required: false
  public get enabled() {
    return this.getBooleanAttribute('enabled') as any;
  }
}
export class DataGoogleContainerClusterPrivateClusterConfig extends cdktf.ComplexComputedList {

  // enable_private_endpoint - computed: true, optional: false, required: false
  public get enablePrivateEndpoint() {
    return this.getBooleanAttribute('enable_private_endpoint') as any;
  }

  // enable_private_nodes - computed: true, optional: false, required: false
  public get enablePrivateNodes() {
    return this.getBooleanAttribute('enable_private_nodes') as any;
  }

  // master_global_access_config - computed: true, optional: false, required: false
  public get masterGlobalAccessConfig() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('master_global_access_config') as any;
  }

  // master_ipv4_cidr_block - computed: true, optional: false, required: false
  public get masterIpv4CidrBlock() {
    return this.getStringAttribute('master_ipv4_cidr_block');
  }

  // peering_name - computed: true, optional: false, required: false
  public get peeringName() {
    return this.getStringAttribute('peering_name');
  }

  // private_endpoint - computed: true, optional: false, required: false
  public get privateEndpoint() {
    return this.getStringAttribute('private_endpoint');
  }

  // public_endpoint - computed: true, optional: false, required: false
  public get publicEndpoint() {
    return this.getStringAttribute('public_endpoint');
  }
}
export class DataGoogleContainerClusterReleaseChannel extends cdktf.ComplexComputedList {

  // channel - computed: true, optional: false, required: false
  public get channel() {
    return this.getStringAttribute('channel');
  }
}
export class DataGoogleContainerClusterResourceUsageExportConfigBigqueryDestination extends cdktf.ComplexComputedList {

  // dataset_id - computed: true, optional: false, required: false
  public get datasetId() {
    return this.getStringAttribute('dataset_id');
  }
}
export class DataGoogleContainerClusterResourceUsageExportConfig extends cdktf.ComplexComputedList {

  // bigquery_destination - computed: true, optional: false, required: false
  public get bigqueryDestination() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('bigquery_destination') as any;
  }

  // enable_network_egress_metering - computed: true, optional: false, required: false
  public get enableNetworkEgressMetering() {
    return this.getBooleanAttribute('enable_network_egress_metering') as any;
  }

  // enable_resource_consumption_metering - computed: true, optional: false, required: false
  public get enableResourceConsumptionMetering() {
    return this.getBooleanAttribute('enable_resource_consumption_metering') as any;
  }
}
export class DataGoogleContainerClusterVerticalPodAutoscaling extends cdktf.ComplexComputedList {

  // enabled - computed: true, optional: false, required: false
  public get enabled() {
    return this.getBooleanAttribute('enabled') as any;
  }
}
export class DataGoogleContainerClusterWorkloadIdentityConfig extends cdktf.ComplexComputedList {

  // workload_pool - computed: true, optional: false, required: false
  public get workloadPool() {
    return this.getStringAttribute('workload_pool');
  }
}

/**
* Represents a {@link https://www.terraform.io/docs/providers/google/d/container_cluster.html google_container_cluster}
*/
export class DataGoogleContainerCluster extends cdktf.TerraformDataSource {

  // =================
  // STATIC PROPERTIES
  // =================
  public static readonly tfResourceType: string = "google_container_cluster";

  // ===========
  // INITIALIZER
  // ===========

  /**
  * Create a new {@link https://www.terraform.io/docs/providers/google/d/container_cluster.html google_container_cluster} Data Source
  *
  * @param scope The scope in which to define this construct
  * @param id The scoped construct ID. Must be unique amongst siblings in the same scope
  * @param options DataGoogleContainerClusterConfig
  */
  public constructor(scope: Construct, id: string, config: DataGoogleContainerClusterConfig) {
    super(scope, id, {
      terraformResourceType: 'google_container_cluster',
      terraformGeneratorMetadata: {
        providerName: 'google'
      },
      provider: config.provider,
      dependsOn: config.dependsOn,
      count: config.count,
      lifecycle: config.lifecycle
    });
    this._location = config.location;
    this._name = config.name;
    this._project = config.project;
  }

  // ==========
  // ATTRIBUTES
  // ==========

  // addons_config - computed: true, optional: false, required: false
  public addonsConfig(index: string) {
    return new DataGoogleContainerClusterAddonsConfig(this, 'addons_config', index);
  }

  // authenticator_groups_config - computed: true, optional: false, required: false
  public authenticatorGroupsConfig(index: string) {
    return new DataGoogleContainerClusterAuthenticatorGroupsConfig(this, 'authenticator_groups_config', index);
  }

  // cluster_autoscaling - computed: true, optional: false, required: false
  public clusterAutoscaling(index: string) {
    return new DataGoogleContainerClusterClusterAutoscaling(this, 'cluster_autoscaling', index);
  }

  // cluster_ipv4_cidr - computed: true, optional: false, required: false
  public get clusterIpv4Cidr() {
    return this.getStringAttribute('cluster_ipv4_cidr');
  }

  // confidential_nodes - computed: true, optional: false, required: false
  public confidentialNodes(index: string) {
    return new DataGoogleContainerClusterConfidentialNodes(this, 'confidential_nodes', index);
  }

  // database_encryption - computed: true, optional: false, required: false
  public databaseEncryption(index: string) {
    return new DataGoogleContainerClusterDatabaseEncryption(this, 'database_encryption', index);
  }

  // datapath_provider - computed: true, optional: false, required: false
  public get datapathProvider() {
    return this.getStringAttribute('datapath_provider');
  }

  // default_max_pods_per_node - computed: true, optional: false, required: false
  public get defaultMaxPodsPerNode() {
    return this.getNumberAttribute('default_max_pods_per_node');
  }

  // default_snat_status - computed: true, optional: false, required: false
  public defaultSnatStatus(index: string) {
    return new DataGoogleContainerClusterDefaultSnatStatus(this, 'default_snat_status', index);
  }

  // description - computed: true, optional: false, required: false
  public get description() {
    return this.getStringAttribute('description');
  }

  // dns_config - computed: true, optional: false, required: false
  public dnsConfig(index: string) {
    return new DataGoogleContainerClusterDnsConfig(this, 'dns_config', index);
  }

  // enable_autopilot - computed: true, optional: false, required: false
  public get enableAutopilot() {
    return this.getBooleanAttribute('enable_autopilot') as any;
  }

  // enable_binary_authorization - computed: true, optional: false, required: false
  public get enableBinaryAuthorization() {
    return this.getBooleanAttribute('enable_binary_authorization') as any;
  }

  // enable_intranode_visibility - computed: true, optional: false, required: false
  public get enableIntranodeVisibility() {
    return this.getBooleanAttribute('enable_intranode_visibility') as any;
  }

  // enable_kubernetes_alpha - computed: true, optional: false, required: false
  public get enableKubernetesAlpha() {
    return this.getBooleanAttribute('enable_kubernetes_alpha') as any;
  }

  // enable_legacy_abac - computed: true, optional: false, required: false
  public get enableLegacyAbac() {
    return this.getBooleanAttribute('enable_legacy_abac') as any;
  }

  // enable_shielded_nodes - computed: true, optional: false, required: false
  public get enableShieldedNodes() {
    return this.getBooleanAttribute('enable_shielded_nodes') as any;
  }

  // enable_tpu - computed: true, optional: false, required: false
  public get enableTpu() {
    return this.getBooleanAttribute('enable_tpu') as any;
  }

  // endpoint - computed: true, optional: false, required: false
  public get endpoint() {
    return this.getStringAttribute('endpoint');
  }

  // id - computed: true, optional: true, required: false
  public get id() {
    return this.getStringAttribute('id');
  }

  // initial_node_count - computed: true, optional: false, required: false
  public get initialNodeCount() {
    return this.getNumberAttribute('initial_node_count');
  }

  // ip_allocation_policy - computed: true, optional: false, required: false
  public ipAllocationPolicy(index: string) {
    return new DataGoogleContainerClusterIpAllocationPolicy(this, 'ip_allocation_policy', index);
  }

  // label_fingerprint - computed: true, optional: false, required: false
  public get labelFingerprint() {
    return this.getStringAttribute('label_fingerprint');
  }

  // location - computed: false, optional: true, required: false
  private _location?: string; 
  public get location() {
    return this.getStringAttribute('location');
  }
  public set location(value: string) {
    this._location = value;
  }
  public resetLocation() {
    this._location = undefined;
  }
  // Temporarily expose input value. Use with caution.
  public get locationInput() {
    return this._location;
  }

  // logging_config - computed: true, optional: false, required: false
  public loggingConfig(index: string) {
    return new DataGoogleContainerClusterLoggingConfig(this, 'logging_config', index);
  }

  // logging_service - computed: true, optional: false, required: false
  public get loggingService() {
    return this.getStringAttribute('logging_service');
  }

  // maintenance_policy - computed: true, optional: false, required: false
  public maintenancePolicy(index: string) {
    return new DataGoogleContainerClusterMaintenancePolicy(this, 'maintenance_policy', index);
  }

  // master_auth - computed: true, optional: false, required: false
  public masterAuth(index: string) {
    return new DataGoogleContainerClusterMasterAuth(this, 'master_auth', index);
  }

  // master_authorized_networks_config - computed: true, optional: false, required: false
  public masterAuthorizedNetworksConfig(index: string) {
    return new DataGoogleContainerClusterMasterAuthorizedNetworksConfig(this, 'master_authorized_networks_config', index);
  }

  // master_version - computed: true, optional: false, required: false
  public get masterVersion() {
    return this.getStringAttribute('master_version');
  }

  // min_master_version - computed: true, optional: false, required: false
  public get minMasterVersion() {
    return this.getStringAttribute('min_master_version');
  }

  // monitoring_config - computed: true, optional: false, required: false
  public monitoringConfig(index: string) {
    return new DataGoogleContainerClusterMonitoringConfig(this, 'monitoring_config', index);
  }

  // monitoring_service - computed: true, optional: false, required: false
  public get monitoringService() {
    return this.getStringAttribute('monitoring_service');
  }

  // name - computed: false, optional: false, required: true
  private _name?: string; 
  public get name() {
    return this.getStringAttribute('name');
  }
  public set name(value: string) {
    this._name = value;
  }
  // Temporarily expose input value. Use with caution.
  public get nameInput() {
    return this._name;
  }

  // network - computed: true, optional: false, required: false
  public get network() {
    return this.getStringAttribute('network');
  }

  // network_policy - computed: true, optional: false, required: false
  public networkPolicy(index: string) {
    return new DataGoogleContainerClusterNetworkPolicy(this, 'network_policy', index);
  }

  // networking_mode - computed: true, optional: false, required: false
  public get networkingMode() {
    return this.getStringAttribute('networking_mode');
  }

  // node_config - computed: true, optional: false, required: false
  public nodeConfig(index: string) {
    return new DataGoogleContainerClusterNodeConfig(this, 'node_config', index);
  }

  // node_locations - computed: true, optional: false, required: false
  public get nodeLocations() {
    return this.getListAttribute('node_locations');
  }

  // node_pool - computed: true, optional: false, required: false
  public nodePool(index: string) {
    return new DataGoogleContainerClusterNodePool(this, 'node_pool', index);
  }

  // node_version - computed: true, optional: false, required: false
  public get nodeVersion() {
    return this.getStringAttribute('node_version');
  }

  // operation - computed: true, optional: false, required: false
  public get operation() {
    return this.getStringAttribute('operation');
  }

  // private_cluster_config - computed: true, optional: false, required: false
  public privateClusterConfig(index: string) {
    return new DataGoogleContainerClusterPrivateClusterConfig(this, 'private_cluster_config', index);
  }

  // private_ipv6_google_access - computed: true, optional: false, required: false
  public get privateIpv6GoogleAccess() {
    return this.getStringAttribute('private_ipv6_google_access');
  }

  // project - computed: false, optional: true, required: false
  private _project?: string; 
  public get project() {
    return this.getStringAttribute('project');
  }
  public set project(value: string) {
    this._project = value;
  }
  public resetProject() {
    this._project = undefined;
  }
  // Temporarily expose input value. Use with caution.
  public get projectInput() {
    return this._project;
  }

  // release_channel - computed: true, optional: false, required: false
  public releaseChannel(index: string) {
    return new DataGoogleContainerClusterReleaseChannel(this, 'release_channel', index);
  }

  // remove_default_node_pool - computed: true, optional: false, required: false
  public get removeDefaultNodePool() {
    return this.getBooleanAttribute('remove_default_node_pool') as any;
  }

  // resource_labels - computed: true, optional: false, required: false
  public resourceLabels(key: string): string {
    return new cdktf.StringMap(this, 'resource_labels').lookup(key);
  }

  // resource_usage_export_config - computed: true, optional: false, required: false
  public resourceUsageExportConfig(index: string) {
    return new DataGoogleContainerClusterResourceUsageExportConfig(this, 'resource_usage_export_config', index);
  }

  // self_link - computed: true, optional: false, required: false
  public get selfLink() {
    return this.getStringAttribute('self_link');
  }

  // services_ipv4_cidr - computed: true, optional: false, required: false
  public get servicesIpv4Cidr() {
    return this.getStringAttribute('services_ipv4_cidr');
  }

  // subnetwork - computed: true, optional: false, required: false
  public get subnetwork() {
    return this.getStringAttribute('subnetwork');
  }

  // tpu_ipv4_cidr_block - computed: true, optional: false, required: false
  public get tpuIpv4CidrBlock() {
    return this.getStringAttribute('tpu_ipv4_cidr_block');
  }

  // vertical_pod_autoscaling - computed: true, optional: false, required: false
  public verticalPodAutoscaling(index: string) {
    return new DataGoogleContainerClusterVerticalPodAutoscaling(this, 'vertical_pod_autoscaling', index);
  }

  // workload_identity_config - computed: true, optional: false, required: false
  public workloadIdentityConfig(index: string) {
    return new DataGoogleContainerClusterWorkloadIdentityConfig(this, 'workload_identity_config', index);
  }

  // =========
  // SYNTHESIS
  // =========

  protected synthesizeAttributes(): { [name: string]: any } {
    return {
      location: cdktf.stringToTerraform(this._location),
      name: cdktf.stringToTerraform(this._name),
      project: cdktf.stringToTerraform(this._project),
    };
  }
}
