// https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html
// generated from terraform resource schema

import { Construct } from 'constructs';
import * as cdktf from 'cdktf';

// Configuration

export interface NetworkConnectivitySpokeConfig extends cdktf.TerraformMetaArguments {
  /**
  * An optional description of the spoke.
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#description NetworkConnectivitySpoke#description}
  */
  readonly description?: string;
  /**
  * Immutable. The URI of the hub that this spoke is attached to.
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#hub NetworkConnectivitySpoke#hub}
  */
  readonly hub: string;
  /**
  * Optional labels in key:value format. For more information about labels, see [Requirements for labels](https://cloud.google.com/resource-manager/docs/creating-managing-labels#requirements).
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#labels NetworkConnectivitySpoke#labels}
  */
  readonly labels?: { [key: string]: string } | cdktf.IResolvable;
  /**
  * The location for the resource
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#location NetworkConnectivitySpoke#location}
  */
  readonly location: string;
  /**
  * Immutable. The name of the spoke. Spoke names must be unique.
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#name NetworkConnectivitySpoke#name}
  */
  readonly name: string;
  /**
  * The project for the resource
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#project NetworkConnectivitySpoke#project}
  */
  readonly project?: string;
  /**
  * linked_interconnect_attachments block
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#linked_interconnect_attachments NetworkConnectivitySpoke#linked_interconnect_attachments}
  */
  readonly linkedInterconnectAttachments?: NetworkConnectivitySpokeLinkedInterconnectAttachments;
  /**
  * linked_router_appliance_instances block
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#linked_router_appliance_instances NetworkConnectivitySpoke#linked_router_appliance_instances}
  */
  readonly linkedRouterApplianceInstances?: NetworkConnectivitySpokeLinkedRouterApplianceInstances;
  /**
  * linked_vpn_tunnels block
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#linked_vpn_tunnels NetworkConnectivitySpoke#linked_vpn_tunnels}
  */
  readonly linkedVpnTunnels?: NetworkConnectivitySpokeLinkedVpnTunnels;
  /**
  * timeouts block
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#timeouts NetworkConnectivitySpoke#timeouts}
  */
  readonly timeouts?: NetworkConnectivitySpokeTimeouts;
}
export interface NetworkConnectivitySpokeLinkedInterconnectAttachments {
  /**
  * A value that controls whether site-to-site data transfer is enabled for these resources. Note that data transfer is available only in supported locations.
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#site_to_site_data_transfer NetworkConnectivitySpoke#site_to_site_data_transfer}
  */
  readonly siteToSiteDataTransfer: boolean | cdktf.IResolvable;
  /**
  * The URIs of linked interconnect attachment resources
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#uris NetworkConnectivitySpoke#uris}
  */
  readonly uris: string[];
}

export function networkConnectivitySpokeLinkedInterconnectAttachmentsToTerraform(struct?: NetworkConnectivitySpokeLinkedInterconnectAttachmentsOutputReference | NetworkConnectivitySpokeLinkedInterconnectAttachments): any {
  if (!cdktf.canInspect(struct)) { return struct; }
  if (cdktf.isComplexElement(struct)) {
    throw new Error("A complex element was used as configuration, this is not supported: https://cdk.tf/complex-object-as-configuration");
  }
  return {
    site_to_site_data_transfer: cdktf.booleanToTerraform(struct!.siteToSiteDataTransfer),
    uris: cdktf.listMapper(cdktf.stringToTerraform)(struct!.uris),
  }
}

export class NetworkConnectivitySpokeLinkedInterconnectAttachmentsOutputReference extends cdktf.ComplexObject {
  private isEmptyObject = false;

  /**
  * @param terraformResource The parent resource
  * @param terraformAttribute The attribute on the parent resource this class is referencing
  * @param isSingleItem True if this is a block, false if it's a list
  */
  public constructor(terraformResource: cdktf.ITerraformResource, terraformAttribute: string, isSingleItem: boolean) {
    super(terraformResource, terraformAttribute, isSingleItem);
  }

  public get internalValue(): NetworkConnectivitySpokeLinkedInterconnectAttachments | undefined {
    let hasAnyValues = this.isEmptyObject;
    const internalValueResult: any = {};
    if (this._siteToSiteDataTransfer) {
      hasAnyValues = true;
      internalValueResult.siteToSiteDataTransfer = this._siteToSiteDataTransfer;
    }
    if (this._uris) {
      hasAnyValues = true;
      internalValueResult.uris = this._uris;
    }
    return hasAnyValues ? internalValueResult : undefined;
  }

  public set internalValue(value: NetworkConnectivitySpokeLinkedInterconnectAttachments | undefined) {
    if (value === undefined) {
      this.isEmptyObject = false;
      this._siteToSiteDataTransfer = undefined;
      this._uris = undefined;
    }
    else {
      this.isEmptyObject = Object.keys(value).length === 0;
      this._siteToSiteDataTransfer = value.siteToSiteDataTransfer;
      this._uris = value.uris;
    }
  }

  // site_to_site_data_transfer - computed: false, optional: false, required: true
  private _siteToSiteDataTransfer?: boolean | cdktf.IResolvable; 
  public get siteToSiteDataTransfer() {
    return this.getBooleanAttribute('site_to_site_data_transfer') as any;
  }
  public set siteToSiteDataTransfer(value: boolean | cdktf.IResolvable) {
    this._siteToSiteDataTransfer = value;
  }
  // Temporarily expose input value. Use with caution.
  public get siteToSiteDataTransferInput() {
    return this._siteToSiteDataTransfer;
  }

  // uris - computed: false, optional: false, required: true
  private _uris?: string[]; 
  public get uris() {
    return this.getListAttribute('uris');
  }
  public set uris(value: string[]) {
    this._uris = value;
  }
  // Temporarily expose input value. Use with caution.
  public get urisInput() {
    return this._uris;
  }
}
export interface NetworkConnectivitySpokeLinkedRouterApplianceInstancesInstances {
  /**
  * The IP address on the VM to use for peering.
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#ip_address NetworkConnectivitySpoke#ip_address}
  */
  readonly ipAddress?: string;
  /**
  * The URI of the virtual machine resource
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#virtual_machine NetworkConnectivitySpoke#virtual_machine}
  */
  readonly virtualMachine?: string;
}

export function networkConnectivitySpokeLinkedRouterApplianceInstancesInstancesToTerraform(struct?: NetworkConnectivitySpokeLinkedRouterApplianceInstancesInstances): any {
  if (!cdktf.canInspect(struct)) { return struct; }
  if (cdktf.isComplexElement(struct)) {
    throw new Error("A complex element was used as configuration, this is not supported: https://cdk.tf/complex-object-as-configuration");
  }
  return {
    ip_address: cdktf.stringToTerraform(struct!.ipAddress),
    virtual_machine: cdktf.stringToTerraform(struct!.virtualMachine),
  }
}

export interface NetworkConnectivitySpokeLinkedRouterApplianceInstances {
  /**
  * A value that controls whether site-to-site data transfer is enabled for these resources. Note that data transfer is available only in supported locations.
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#site_to_site_data_transfer NetworkConnectivitySpoke#site_to_site_data_transfer}
  */
  readonly siteToSiteDataTransfer: boolean | cdktf.IResolvable;
  /**
  * instances block
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#instances NetworkConnectivitySpoke#instances}
  */
  readonly instances: NetworkConnectivitySpokeLinkedRouterApplianceInstancesInstances[];
}

export function networkConnectivitySpokeLinkedRouterApplianceInstancesToTerraform(struct?: NetworkConnectivitySpokeLinkedRouterApplianceInstancesOutputReference | NetworkConnectivitySpokeLinkedRouterApplianceInstances): any {
  if (!cdktf.canInspect(struct)) { return struct; }
  if (cdktf.isComplexElement(struct)) {
    throw new Error("A complex element was used as configuration, this is not supported: https://cdk.tf/complex-object-as-configuration");
  }
  return {
    site_to_site_data_transfer: cdktf.booleanToTerraform(struct!.siteToSiteDataTransfer),
    instances: cdktf.listMapper(networkConnectivitySpokeLinkedRouterApplianceInstancesInstancesToTerraform)(struct!.instances),
  }
}

export class NetworkConnectivitySpokeLinkedRouterApplianceInstancesOutputReference extends cdktf.ComplexObject {
  private isEmptyObject = false;

  /**
  * @param terraformResource The parent resource
  * @param terraformAttribute The attribute on the parent resource this class is referencing
  * @param isSingleItem True if this is a block, false if it's a list
  */
  public constructor(terraformResource: cdktf.ITerraformResource, terraformAttribute: string, isSingleItem: boolean) {
    super(terraformResource, terraformAttribute, isSingleItem);
  }

  public get internalValue(): NetworkConnectivitySpokeLinkedRouterApplianceInstances | undefined {
    let hasAnyValues = this.isEmptyObject;
    const internalValueResult: any = {};
    if (this._siteToSiteDataTransfer) {
      hasAnyValues = true;
      internalValueResult.siteToSiteDataTransfer = this._siteToSiteDataTransfer;
    }
    if (this._instances) {
      hasAnyValues = true;
      internalValueResult.instances = this._instances;
    }
    return hasAnyValues ? internalValueResult : undefined;
  }

  public set internalValue(value: NetworkConnectivitySpokeLinkedRouterApplianceInstances | undefined) {
    if (value === undefined) {
      this.isEmptyObject = false;
      this._siteToSiteDataTransfer = undefined;
      this._instances = undefined;
    }
    else {
      this.isEmptyObject = Object.keys(value).length === 0;
      this._siteToSiteDataTransfer = value.siteToSiteDataTransfer;
      this._instances = value.instances;
    }
  }

  // site_to_site_data_transfer - computed: false, optional: false, required: true
  private _siteToSiteDataTransfer?: boolean | cdktf.IResolvable; 
  public get siteToSiteDataTransfer() {
    return this.getBooleanAttribute('site_to_site_data_transfer') as any;
  }
  public set siteToSiteDataTransfer(value: boolean | cdktf.IResolvable) {
    this._siteToSiteDataTransfer = value;
  }
  // Temporarily expose input value. Use with caution.
  public get siteToSiteDataTransferInput() {
    return this._siteToSiteDataTransfer;
  }

  // instances - computed: false, optional: false, required: true
  private _instances?: NetworkConnectivitySpokeLinkedRouterApplianceInstancesInstances[]; 
  public get instances() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('instances') as any;
  }
  public set instances(value: NetworkConnectivitySpokeLinkedRouterApplianceInstancesInstances[]) {
    this._instances = value;
  }
  // Temporarily expose input value. Use with caution.
  public get instancesInput() {
    return this._instances;
  }
}
export interface NetworkConnectivitySpokeLinkedVpnTunnels {
  /**
  * A value that controls whether site-to-site data transfer is enabled for these resources. Note that data transfer is available only in supported locations.
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#site_to_site_data_transfer NetworkConnectivitySpoke#site_to_site_data_transfer}
  */
  readonly siteToSiteDataTransfer: boolean | cdktf.IResolvable;
  /**
  * The URIs of linked VPN tunnel resources.
  * 
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#uris NetworkConnectivitySpoke#uris}
  */
  readonly uris: string[];
}

export function networkConnectivitySpokeLinkedVpnTunnelsToTerraform(struct?: NetworkConnectivitySpokeLinkedVpnTunnelsOutputReference | NetworkConnectivitySpokeLinkedVpnTunnels): any {
  if (!cdktf.canInspect(struct)) { return struct; }
  if (cdktf.isComplexElement(struct)) {
    throw new Error("A complex element was used as configuration, this is not supported: https://cdk.tf/complex-object-as-configuration");
  }
  return {
    site_to_site_data_transfer: cdktf.booleanToTerraform(struct!.siteToSiteDataTransfer),
    uris: cdktf.listMapper(cdktf.stringToTerraform)(struct!.uris),
  }
}

export class NetworkConnectivitySpokeLinkedVpnTunnelsOutputReference extends cdktf.ComplexObject {
  private isEmptyObject = false;

  /**
  * @param terraformResource The parent resource
  * @param terraformAttribute The attribute on the parent resource this class is referencing
  * @param isSingleItem True if this is a block, false if it's a list
  */
  public constructor(terraformResource: cdktf.ITerraformResource, terraformAttribute: string, isSingleItem: boolean) {
    super(terraformResource, terraformAttribute, isSingleItem);
  }

  public get internalValue(): NetworkConnectivitySpokeLinkedVpnTunnels | undefined {
    let hasAnyValues = this.isEmptyObject;
    const internalValueResult: any = {};
    if (this._siteToSiteDataTransfer) {
      hasAnyValues = true;
      internalValueResult.siteToSiteDataTransfer = this._siteToSiteDataTransfer;
    }
    if (this._uris) {
      hasAnyValues = true;
      internalValueResult.uris = this._uris;
    }
    return hasAnyValues ? internalValueResult : undefined;
  }

  public set internalValue(value: NetworkConnectivitySpokeLinkedVpnTunnels | undefined) {
    if (value === undefined) {
      this.isEmptyObject = false;
      this._siteToSiteDataTransfer = undefined;
      this._uris = undefined;
    }
    else {
      this.isEmptyObject = Object.keys(value).length === 0;
      this._siteToSiteDataTransfer = value.siteToSiteDataTransfer;
      this._uris = value.uris;
    }
  }

  // site_to_site_data_transfer - computed: false, optional: false, required: true
  private _siteToSiteDataTransfer?: boolean | cdktf.IResolvable; 
  public get siteToSiteDataTransfer() {
    return this.getBooleanAttribute('site_to_site_data_transfer') as any;
  }
  public set siteToSiteDataTransfer(value: boolean | cdktf.IResolvable) {
    this._siteToSiteDataTransfer = value;
  }
  // Temporarily expose input value. Use with caution.
  public get siteToSiteDataTransferInput() {
    return this._siteToSiteDataTransfer;
  }

  // uris - computed: false, optional: false, required: true
  private _uris?: string[]; 
  public get uris() {
    return this.getListAttribute('uris');
  }
  public set uris(value: string[]) {
    this._uris = value;
  }
  // Temporarily expose input value. Use with caution.
  public get urisInput() {
    return this._uris;
  }
}
export interface NetworkConnectivitySpokeTimeouts {
  /**
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#create NetworkConnectivitySpoke#create}
  */
  readonly create?: string;
  /**
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#delete NetworkConnectivitySpoke#delete}
  */
  readonly delete?: string;
  /**
  * Docs at Terraform Registry: {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html#update NetworkConnectivitySpoke#update}
  */
  readonly update?: string;
}

export function networkConnectivitySpokeTimeoutsToTerraform(struct?: NetworkConnectivitySpokeTimeoutsOutputReference | NetworkConnectivitySpokeTimeouts): any {
  if (!cdktf.canInspect(struct)) { return struct; }
  if (cdktf.isComplexElement(struct)) {
    throw new Error("A complex element was used as configuration, this is not supported: https://cdk.tf/complex-object-as-configuration");
  }
  return {
    create: cdktf.stringToTerraform(struct!.create),
    delete: cdktf.stringToTerraform(struct!.delete),
    update: cdktf.stringToTerraform(struct!.update),
  }
}

export class NetworkConnectivitySpokeTimeoutsOutputReference extends cdktf.ComplexObject {
  private isEmptyObject = false;

  /**
  * @param terraformResource The parent resource
  * @param terraformAttribute The attribute on the parent resource this class is referencing
  * @param isSingleItem True if this is a block, false if it's a list
  */
  public constructor(terraformResource: cdktf.ITerraformResource, terraformAttribute: string, isSingleItem: boolean) {
    super(terraformResource, terraformAttribute, isSingleItem);
  }

  public get internalValue(): NetworkConnectivitySpokeTimeouts | undefined {
    let hasAnyValues = this.isEmptyObject;
    const internalValueResult: any = {};
    if (this._create) {
      hasAnyValues = true;
      internalValueResult.create = this._create;
    }
    if (this._delete) {
      hasAnyValues = true;
      internalValueResult.delete = this._delete;
    }
    if (this._update) {
      hasAnyValues = true;
      internalValueResult.update = this._update;
    }
    return hasAnyValues ? internalValueResult : undefined;
  }

  public set internalValue(value: NetworkConnectivitySpokeTimeouts | undefined) {
    if (value === undefined) {
      this.isEmptyObject = false;
      this._create = undefined;
      this._delete = undefined;
      this._update = undefined;
    }
    else {
      this.isEmptyObject = Object.keys(value).length === 0;
      this._create = value.create;
      this._delete = value.delete;
      this._update = value.update;
    }
  }

  // create - computed: false, optional: true, required: false
  private _create?: string; 
  public get create() {
    return this.getStringAttribute('create');
  }
  public set create(value: string) {
    this._create = value;
  }
  public resetCreate() {
    this._create = undefined;
  }
  // Temporarily expose input value. Use with caution.
  public get createInput() {
    return this._create;
  }

  // delete - computed: false, optional: true, required: false
  private _delete?: string; 
  public get delete() {
    return this.getStringAttribute('delete');
  }
  public set delete(value: string) {
    this._delete = value;
  }
  public resetDelete() {
    this._delete = undefined;
  }
  // Temporarily expose input value. Use with caution.
  public get deleteInput() {
    return this._delete;
  }

  // update - computed: false, optional: true, required: false
  private _update?: string; 
  public get update() {
    return this.getStringAttribute('update');
  }
  public set update(value: string) {
    this._update = value;
  }
  public resetUpdate() {
    this._update = undefined;
  }
  // Temporarily expose input value. Use with caution.
  public get updateInput() {
    return this._update;
  }
}

/**
* Represents a {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html google_network_connectivity_spoke}
*/
export class NetworkConnectivitySpoke extends cdktf.TerraformResource {

  // =================
  // STATIC PROPERTIES
  // =================
  public static readonly tfResourceType: string = "google_network_connectivity_spoke";

  // ===========
  // INITIALIZER
  // ===========

  /**
  * Create a new {@link https://www.terraform.io/docs/providers/google/r/network_connectivity_spoke.html google_network_connectivity_spoke} Resource
  *
  * @param scope The scope in which to define this construct
  * @param id The scoped construct ID. Must be unique amongst siblings in the same scope
  * @param options NetworkConnectivitySpokeConfig
  */
  public constructor(scope: Construct, id: string, config: NetworkConnectivitySpokeConfig) {
    super(scope, id, {
      terraformResourceType: 'google_network_connectivity_spoke',
      terraformGeneratorMetadata: {
        providerName: 'google'
      },
      provider: config.provider,
      dependsOn: config.dependsOn,
      count: config.count,
      lifecycle: config.lifecycle
    });
    this._description = config.description;
    this._hub = config.hub;
    this._labels = config.labels;
    this._location = config.location;
    this._name = config.name;
    this._project = config.project;
    this._linkedInterconnectAttachments.internalValue = config.linkedInterconnectAttachments;
    this._linkedRouterApplianceInstances.internalValue = config.linkedRouterApplianceInstances;
    this._linkedVpnTunnels.internalValue = config.linkedVpnTunnels;
    this._timeouts.internalValue = config.timeouts;
  }

  // ==========
  // ATTRIBUTES
  // ==========

  // create_time - computed: true, optional: false, required: false
  public get createTime() {
    return this.getStringAttribute('create_time');
  }

  // description - computed: false, optional: true, required: false
  private _description?: string; 
  public get description() {
    return this.getStringAttribute('description');
  }
  public set description(value: string) {
    this._description = value;
  }
  public resetDescription() {
    this._description = undefined;
  }
  // Temporarily expose input value. Use with caution.
  public get descriptionInput() {
    return this._description;
  }

  // hub - computed: false, optional: false, required: true
  private _hub?: string; 
  public get hub() {
    return this.getStringAttribute('hub');
  }
  public set hub(value: string) {
    this._hub = value;
  }
  // Temporarily expose input value. Use with caution.
  public get hubInput() {
    return this._hub;
  }

  // id - computed: true, optional: true, required: false
  public get id() {
    return this.getStringAttribute('id');
  }

  // labels - computed: false, optional: true, required: false
  private _labels?: { [key: string]: string } | cdktf.IResolvable; 
  public get labels() {
    // Getting the computed value is not yet implemented
    return this.interpolationForAttribute('labels') as any;
  }
  public set labels(value: { [key: string]: string } | cdktf.IResolvable) {
    this._labels = value;
  }
  public resetLabels() {
    this._labels = undefined;
  }
  // Temporarily expose input value. Use with caution.
  public get labelsInput() {
    return this._labels;
  }

  // location - computed: false, optional: false, required: true
  private _location?: string; 
  public get location() {
    return this.getStringAttribute('location');
  }
  public set location(value: string) {
    this._location = value;
  }
  // Temporarily expose input value. Use with caution.
  public get locationInput() {
    return this._location;
  }

  // name - computed: false, optional: false, required: true
  private _name?: string; 
  public get name() {
    return this.getStringAttribute('name');
  }
  public set name(value: string) {
    this._name = value;
  }
  // Temporarily expose input value. Use with caution.
  public get nameInput() {
    return this._name;
  }

  // project - computed: true, optional: true, required: false
  private _project?: string; 
  public get project() {
    return this.getStringAttribute('project');
  }
  public set project(value: string) {
    this._project = value;
  }
  public resetProject() {
    this._project = undefined;
  }
  // Temporarily expose input value. Use with caution.
  public get projectInput() {
    return this._project;
  }

  // state - computed: true, optional: false, required: false
  public get state() {
    return this.getStringAttribute('state');
  }

  // unique_id - computed: true, optional: false, required: false
  public get uniqueId() {
    return this.getStringAttribute('unique_id');
  }

  // update_time - computed: true, optional: false, required: false
  public get updateTime() {
    return this.getStringAttribute('update_time');
  }

  // linked_interconnect_attachments - computed: false, optional: true, required: false
  private _linkedInterconnectAttachments = new NetworkConnectivitySpokeLinkedInterconnectAttachmentsOutputReference(this as any, "linked_interconnect_attachments", true);
  public get linkedInterconnectAttachments() {
    return this._linkedInterconnectAttachments;
  }
  public putLinkedInterconnectAttachments(value: NetworkConnectivitySpokeLinkedInterconnectAttachments) {
    this._linkedInterconnectAttachments.internalValue = value;
  }
  public resetLinkedInterconnectAttachments() {
    this._linkedInterconnectAttachments.internalValue = undefined;
  }
  // Temporarily expose input value. Use with caution.
  public get linkedInterconnectAttachmentsInput() {
    return this._linkedInterconnectAttachments.internalValue;
  }

  // linked_router_appliance_instances - computed: false, optional: true, required: false
  private _linkedRouterApplianceInstances = new NetworkConnectivitySpokeLinkedRouterApplianceInstancesOutputReference(this as any, "linked_router_appliance_instances", true);
  public get linkedRouterApplianceInstances() {
    return this._linkedRouterApplianceInstances;
  }
  public putLinkedRouterApplianceInstances(value: NetworkConnectivitySpokeLinkedRouterApplianceInstances) {
    this._linkedRouterApplianceInstances.internalValue = value;
  }
  public resetLinkedRouterApplianceInstances() {
    this._linkedRouterApplianceInstances.internalValue = undefined;
  }
  // Temporarily expose input value. Use with caution.
  public get linkedRouterApplianceInstancesInput() {
    return this._linkedRouterApplianceInstances.internalValue;
  }

  // linked_vpn_tunnels - computed: false, optional: true, required: false
  private _linkedVpnTunnels = new NetworkConnectivitySpokeLinkedVpnTunnelsOutputReference(this as any, "linked_vpn_tunnels", true);
  public get linkedVpnTunnels() {
    return this._linkedVpnTunnels;
  }
  public putLinkedVpnTunnels(value: NetworkConnectivitySpokeLinkedVpnTunnels) {
    this._linkedVpnTunnels.internalValue = value;
  }
  public resetLinkedVpnTunnels() {
    this._linkedVpnTunnels.internalValue = undefined;
  }
  // Temporarily expose input value. Use with caution.
  public get linkedVpnTunnelsInput() {
    return this._linkedVpnTunnels.internalValue;
  }

  // timeouts - computed: false, optional: true, required: false
  private _timeouts = new NetworkConnectivitySpokeTimeoutsOutputReference(this as any, "timeouts", true);
  public get timeouts() {
    return this._timeouts;
  }
  public putTimeouts(value: NetworkConnectivitySpokeTimeouts) {
    this._timeouts.internalValue = value;
  }
  public resetTimeouts() {
    this._timeouts.internalValue = undefined;
  }
  // Temporarily expose input value. Use with caution.
  public get timeoutsInput() {
    return this._timeouts.internalValue;
  }

  // =========
  // SYNTHESIS
  // =========

  protected synthesizeAttributes(): { [name: string]: any } {
    return {
      description: cdktf.stringToTerraform(this._description),
      hub: cdktf.stringToTerraform(this._hub),
      labels: cdktf.hashMapper(cdktf.anyToTerraform)(this._labels),
      location: cdktf.stringToTerraform(this._location),
      name: cdktf.stringToTerraform(this._name),
      project: cdktf.stringToTerraform(this._project),
      linked_interconnect_attachments: networkConnectivitySpokeLinkedInterconnectAttachmentsToTerraform(this._linkedInterconnectAttachments.internalValue),
      linked_router_appliance_instances: networkConnectivitySpokeLinkedRouterApplianceInstancesToTerraform(this._linkedRouterApplianceInstances.internalValue),
      linked_vpn_tunnels: networkConnectivitySpokeLinkedVpnTunnelsToTerraform(this._linkedVpnTunnels.internalValue),
      timeouts: networkConnectivitySpokeTimeoutsToTerraform(this._timeouts.internalValue),
    };
  }
}
