import { Construct } from "constructs";
import { App, TerraformStack, GcsBackend } from "cdktf";
import {
  Project,
  GoogleProvider,
} from "@cdktf/provider-google";
import {
  providerConfig,
  backendConfig,
  orgConfig,
} from "./conf/gcp";
import { GcpNetworkingConfig} from './networking/gcp_config';
import { GcpNetwork } from './networking/gcp';

class MyStack extends TerraformStack {
  private readonly project: Project;
  constructor(scope: Construct, name: string) {
    super(scope, name);

    new GoogleProvider(this, "example-main-provider", providerConfig);
    this.project = new Project(this, "example-main-project", orgConfig);
    const net = new GcpNetworkingConfig(this.project);
    new GcpNetwork(this, "main", net)
  }
}


const app = new App();
const stack = new MyStack(app, "main");

new GcsBackend(stack, backendConfig);

app.synth();
